<?php

namespace Ivoz\Provider\Application\Service\Invoice;

use Ivoz\Core\Application\DataTransferObjectInterface;
use Ivoz\Core\Application\Service\StoragePathResolverCollection;
use Ivoz\Core\Domain\Model\EntityInterface;
use Ivoz\Core\Application\Service\Assembler\CustomDtoAssemblerInterface;
use Ivoz\Provider\Domain\Model\Invoice\InvoiceDto;
use Ivoz\Provider\Domain\Model\Invoice\InvoiceInterface;
use Assert\Assertion;

class InvoiceDtoAssembler implements CustomDtoAssemblerInterface
{
    protected $storagePathResolver;

    public function __construct(
        StoragePathResolverCollection $storagePathResolver
    ) {
        $this->storagePathResolver = $storagePathResolver;
    }

    /**
     * @param InvoiceInterface $invoice
     * @throws \Exception
     */
    public function toDto(EntityInterface $invoice, int $depth = 0, string $context = null): DataTransferObjectInterface
    {
        Assertion::isInstanceOf($invoice, InvoiceInterface::class);

        /** @var InvoiceDto $dto */
        $dto = $invoice->toDto($depth);
        $id = $invoice->getId();

        if (!$id) {
            return $dto;
        }

        if ($invoice->getPdf()->getFileSize()) {
            $pathResolver = $this
                ->storagePathResolver
                ->getPathResolver('Pdf');

            $dto->setPdfPath(
                $pathResolver->getFilePath($invoice)
            );
        }

        return $dto;
    }
}
