<?php

namespace Ivoz\Provider\Infrastructure\Persistence\Doctrine;

use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Ivoz\Core\Infrastructure\Persistence\Doctrine\Model\Helper\CriteriaHelper;
use Ivoz\Provider\Domain\Model\CallCsvScheduler\CallCsvSchedulerInterface;
use Ivoz\Provider\Domain\Model\CallCsvScheduler\CallCsvSchedulerRepository;
use Ivoz\Provider\Domain\Model\CallCsvScheduler\CallCsvScheduler;
use Ivoz\Provider\Infrastructure\Persistence\Doctrine\Traits\CountByCriteriaTrait;
use Symfony\Bridge\Doctrine\RegistryInterface;

/**
 * CallCsvSchedulerDoctrineRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class CallCsvSchedulerDoctrineRepository extends ServiceEntityRepository implements CallCsvSchedulerRepository
{
    use CountByCriteriaTrait;

    public function __construct(RegistryInterface $registry)
    {
        parent::__construct($registry, CallCsvScheduler::class);
    }

    /**
     * @inheritdoc
     * @see CallCsvSchedulerRepository::getCompanyIdsInUse
     */
    public function getCompanyIdsInUse($schedulerIdToExclude)
    {
        if (!$schedulerIdToExclude) {
            $schedulerIdToExclude = -1;
        }

        $qb = $this
            ->createQueryBuilder('self');

        $query = $qb
            ->select('IDENTITY(self.company)')
            ->where(
                $qb->expr()->neq('self.id', $schedulerIdToExclude)
            );

        $ids = [];
        $items = $query
            ->getQuery()
            ->getScalarResult();

        foreach ($items as $item) {
            $ids[] = current($item);
        }

        return $ids;
    }

    /**
     * @inheritdoc
     * @see CallCsvSchedulerRepository::getPendingSchedulers
     */
    public function getPendingSchedulers()
    {
        $now = new \DateTime(
            null,
            new \DateTimeZone('UTC')
        );

        // nextExecution
        $qb = $this->createQueryBuilder('self');
        $nextExecutionCondition =
            $qb
                ->expr()
                ->lte(
                    'self.nextExecution',
                    "'" . $now->format('Y-m-d H:i:s') . "'"
                );

        $query = $qb
            ->select('self')
            ->where($nextExecutionCondition)
            ->getQuery();

        return $query->getResult();
    }

    /**
     * @param CallCsvSchedulerInterface $callCsvScheduler
     * @return bool
     */
    public function hasUniqueName(CallCsvSchedulerInterface $callCsvScheduler)
    {

        $criteria = CriteriaHelper::fromArray([
            ['id', 'neq', $callCsvScheduler->getId()],
            ['company', 'eq', $callCsvScheduler->getCompany()->getId()],
            ['name', 'eq', $callCsvScheduler->getName()]
        ]);

        $result = $this->countByCriteria($criteria);

        return $result === 0;
    }
}
