<?php

namespace Ivoz\Provider\Infrastructure\Persistence\Doctrine;

use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Ivoz\Core\Infrastructure\Persistence\Doctrine\Model\Helper\CriteriaHelper;
use Ivoz\Provider\Domain\Model\WebPortal\WebPortalRepository;
use Ivoz\Provider\Domain\Model\WebPortal\WebPortalInterface;
use Ivoz\Provider\Domain\Model\WebPortal\WebPortal;
use Symfony\Bridge\Doctrine\RegistryInterface;

/**
 * WebPortalDoctrineRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class WebPortalDoctrineRepository extends ServiceEntityRepository implements WebPortalRepository
{
    public function __construct(RegistryInterface $registry)
    {
        parent::__construct($registry, WebPortal::class);
    }

    /**
     * @param string $serverName
     * @return WebPortalInterface | null
     */
    public function findUserUrlByServerName(string $serverName)
    {
        $conditions = [
            'url' => 'https://' . $serverName,
            'urlType' => 'user'
        ];
        /** @var WebPortalInterface $response */
        $response = $this->findOneBy($conditions);

        return $response;
    }

    /**
     * @param string $url
     * @return int | null
     * @throws \DomainException
     */
    public function findBrandIdByUrl(string $url)
    {
        $criteria = CriteriaHelper::fromArray([
            ['url', 'eq',  $url]
        ]);

        $qb = $this
            ->createQueryBuilder('self')
            ->select('IDENTITY(self.brand)')
            ->addCriteria($criteria);

        try {
            $brandId = $qb
                ->getQuery()
                ->getSingleScalarResult();
        } catch (\Exception $e) {
            throw new \DomainException(
                'Could not resolve WebPortal by request URL: ' . $url,
                0,
                $e
            );
        }

        if (is_null($brandId)) {
            return null;
        }

        return (int) $brandId;
    }
}
